unit maus;
{      Alle Mausfunktionen werden ber den Interrupt 33H angesteuert. Die
       Parameterbergabe erfolgt in den CPU-Registern. AX enthlt die
       Nummer der auszufhrenden Funktion. Sind weitere Parameter ntig,
       werden sie in bx, cx, dx, si, di und es bergeben. Alle Register,
       die keine Rckgabewerte enthalten, bleiben whrend des Interrupts
       erhalten. Es gibt folgende Funktionen:

       Funktion    Bedeutung
       -------------------------------------------------------------
       AX = 0  Initialisierung der Maus
               Rckgabe: AX = -1 (Maus ist vorhanden)
                         BX = 3 (Zahl der Tasten)

       AX = 1  Mauscursor sichtbar schalten

       AX = 2  Mauscursor unsichtbar schalten

       AX = 3  Tastenstatus und Position auslesen
               Rckgabe: BX Tastenstatus
                          (Bits 0/1/2: links/rechts/mitte,1=gedrckt)
                       CX x-position
                       DX y-position

       AX = 4  Mausposition setzen
               CX = neue x-Position
               DX = neue y-Position (Angaben immer in Pixeln, auch
                                       in den Textmodi!)

       AX = 5  Auslesen der 'Button Press Info'
               BX = Nummer der Taste (0=links, 1=rechts, 2=mitte)
               Rckgabe: BX = Anzahl der Tastendrcke
                       CX = x-Position des letzten Drckens
                       DX = y-Position des letzten Drckens

       AX = 6  Auslesen der 'Button Release Info'
               BX = Nummer der Taste (0=links, 1=rechts, 2=mitte)
               Rckgabe: BX = Anzahl des Tastenlsens
                       CX = x-Position des letzten Lsens
                       DX = y-Position des letzten Lsens

       AX = 7  Minimale/Maximale x-Position setzen
               CX = Minimum
               DX = Maximum (Der Mauscursor bewegt sich nur zwischen
                               diesen Werten)

       AX = 8  Minimale/Maximale y-Position setzen
               CX = Minimum
               DX = Maximum

       AX = 9  Grafischen Cursor definieren
               BX = relative x-Position
               CX = relative y-Position
               ES:DX = Zeiger auf neuen Cursorblock
                   (Zunchst 16 Worte mit Cursormaske, dann
                    16 Worte mit den Cursorwerten)

       AX = 10 Text Cursor definieren
               BX = 0: Software Cursor, 1: Hardware Cursor
               CX = Software: Bildschirmmaske,Hardware: scan line start
               DX = Software: Bildschirmwert, Hardware: scan line end

       AX = 11 Bewegungswerte lesen
               Rckgabe: CX = akkumulierter x-Zhler
                       DX = akkumulierter y-Zhler
               Gezhlt werden 'Mickeys' (1/200 Zoll). Nach Aufruf
               werden die Zhler auf Null gesetzt

       AX = 12 Interrupt Maske und Einsprung definieren
               CX = Maske
               ES:DX = Interrupt Einsprung
               Die Interruptroutine muss mit 'RETF' (nicht IRET!)
               abgeschlossen sein. Maske:
               Bit 0:  1:Cursor hat sich bewegt
                   1:  1:linke Taste gedrckt
                   2:  1:linke Taste gelst
                   3:  1:rechte Taste gedrckt
                   4:  1:rechte Taste gelst
                   5:  1:mittlere Taste gedrckt
                   6:  1:mittlere Taste gelst
               Ist das entsprechende Bit in der Maske 1, wird die
               Routine beim Auftreten der Bedingung angefahren.
               Beim Einsprung in die Routine haben die Register

               folgende Belegung:
               AX: Bits, warum Routine gestartet wird, Belegung s.o.
               BX: Tastenstatus
               CX: x-Position
               DX: y-Position
               SI: akkumulierter x-Zhler
               DI: akkumulierter y-Zhler
               (Hinweis: zum Abschalten der Interruptroutine, Funktion
               12 mit CX = 0 aufrufen)

       AX = 13 Light-Pen Emulation ein
               Die Funktionen des Lichtgriffels werden auf die Maus
               abgebildet

       AX = 14 Light-Pen Emulation aus
               Die Emulation wird wieder ausgeschaltet

       AX = 15 Mickey/Pixel Rate setzen
               CX = x-Rate (n Mickeys pro 8 Pixel in x-Richtung)
               DX = y-Rate (n Mickeys pro 8 Pixel in y-Richtung)
               Legt fest, wie schnell die Maus ber den Bildschirm
               bewegt wird.

       AX = 16 Bedingtes Ausschalten
               CX = linke Seite
               DX = obere Seite
               SI = rechte Seite
               DI = untere Seite (alles in Pixeln)
               Sobald der Cursor in diesen Bereich bewegt wird, wird er
               ausgeschaltet. Ntzlich zum Bildaufbau.  Wieder
               Einschalten durch Funktion 1.

       AX = 19 Doppelte Geschwindigkeit setzen
               DX = Mickeys/Sekunde
               wird diese Geschwindigkeit berschritten, bewegt sich
               der Cursor doppelt so schnell ber den Bildschirm
               Zum Ausschalten: Sehr grossen Wert in CX bergeben

       AX = 20 Interrupt Routine austauschen
               BX: Segment der neuen Routine
               CX: neue Maske
               DX: Offset der neuen Routine (siehe Funktion 12)

               Rckgabe: BX: Segment der alten Routine
                       CX: alte Maske
                       DX: Offset der alten Routine

       AX = 29 Seitennummer des Bildschirms fr Cursor setzen
               BX = neue Seite
               Wenn der Bildschirmadapter mehr als eine Seite speichern
               kann, wird hier bestimmt, auf welcher Seite der Maus-
               cursor erscheint.

       AX = 30 Seitennummer auslesen
               Rckgabe: BX = Seitennummer fr Mauscursor

       Achtung!!

       Es werden keinerlei Prfungen der Register vorgenommen, falsche
       Werte knnen das Programm zu unvorhersagbaren Reaktionen
       veranlassen.}

interface
uses dos;

procedure mausinit(var tasten:word);
procedure maussicht;
procedure mausunsicht;
procedure maustastpos(var tasten,xpos,ypos:word);
procedure maussetpos(x,y:word);
procedure mausbuttonpressinfo(var taste,x,y:word);
procedure mausbuttonreleaseinfo(var taste,x,y:word);
procedure mausminmaxx(minimum,maximum:word);
procedure mausminmaxy(minimum,maximum:word);
procedure mausgrafikcursor(relx,rely:word;var zeiger);
procedure maustextcursor(hartsoft,maske_end,wert_start:word);
procedure mausbewegung(var xzahler,yzahler:word);
procedure mausinterrupt(maske:word;einsprung:pointer);
procedure mauslightpenon;
procedure mauslightpenoff;
procedure mausgeschwindigkeit(xrate,yrate:word);
procedure mausbedingtesaus(links,oben,rechts,unten:word);
procedure mausdoppelt(abwann:word);
procedure mausinterruptaustausch(var maske:word;neu:pointer);
procedure maussetseite(seite:word);
function mausgetseite:word;
function mauskeypressed:boolean;
function mausversion:word;
function mausinxy(x1,y1,x2,y2:integer):boolean;

implementation
var mausreg:registers;

procedure mausinit(var tasten:word);
begin
  with mausreg do begin
    ax:=0;
    intr($33,mausreg);
    if integer(ax)=-1 then
    tasten:=integer(bx) else tasten:=0;
  end;
end;

procedure maussicht;
begin
  with mausreg do begin
    ax:=1;
    intr($33,mausreg);
  end;
end;

procedure mausunsicht;
begin
  with mausreg do begin
    ax:=2;
    intr($33,mausreg);
  end;
end;

procedure maustastpos(var tasten,xpos,ypos:word);
begin
  with mausreg do begin
    ax:=3;
    intr($33,mausreg);
    tasten:=bx;
    xpos:=cx;
    ypos:=dx;
  end;
end;

procedure maussetpos(x,y:word);
begin
  with mausreg do begin
    ax:=4;
    cx:=x;
    dx:=y;
    intr($33,mausreg);
  end;
end;

procedure mausbuttonpressinfo(var taste,x,y:word);
begin
  with mausreg do begin
    ax:=5;
    bx:=taste;
    intr($33,mausreg);
    taste:=bx;
    x:=cx;
    y:=dx;
  end;
end;

procedure mausbuttonreleaseinfo(var taste,x,y:word);
begin
  with mausreg do begin
    ax:=6;
    bx:=taste;
    intr($33,mausreg);
    taste:=bx;
    x:=cx;
    y:=dx;
  end;
end;

procedure mausminmaxx(minimum,maximum:word);
begin
  with mausreg do begin
    ax:=7;
    cx:=minimum;
    dx:=maximum;
    intr($33,mausreg);
  end;
end;

procedure mausminmaxy(minimum,maximum:word);
begin
  with mausreg do begin
    ax:=8;
    cx:=minimum;
    dx:=maximum;
    intr($33,mausreg);
  end;
end;

procedure mausgrafikcursor(relx,rely:word;var zeiger);
begin
  with mausreg do begin
    ax:=9;
    bx:=relx;
    cx:=rely;
    es:=seg(zeiger);
    dx:=ofs(zeiger);
    intr($33,mausreg);
  end;
end;

procedure maustextcursor(hartsoft,maske_end,wert_start:word);
begin
  with mausreg do begin
    ax:=10;
    bx:=hartsoft;
    cx:=maske_end;
    dx:=wert_start;
    intr($33,mausreg);
  end;
end;

procedure mausbewegung(var xzahler,yzahler:word);
begin
  with mausreg do begin
    ax:=11;
    intr($33,mausreg);
    xzahler:=cx;
    yzahler:=dx;
  end;
end;

procedure mausinterrupt(maske:word;einsprung:pointer);
begin
  with mausreg do begin
    ax:=12;
    cx:=maske;
    es:=seg(einsprung^);
    dx:=ofs(einsprung^);
    intr($33,mausreg);
  end;
end;

procedure mauslightpenon;
begin
  with mausreg do begin
    ax:=13;
    intr($33,mausreg);
  end;
end;

procedure mauslightpenoff;
begin
  with mausreg do begin
    ax:=14;
    intr($33,mausreg);
  end;
end;

procedure mausgeschwindigkeit(xrate,yrate:word);
begin
  with mausreg do begin
    ax:=15;
    cx:=xrate;
    dx:=yrate;
    intr($33,mausreg);
  end;
end;

procedure mausbedingtesaus(links,oben,rechts,unten:word);
begin
  with mausreg do begin
    ax:=16;
    cx:=links;
    dx:=oben;
    si:=rechts;
    di:=unten;
    intr($33,mausreg);
  end;
end;

procedure mausdoppelt(abwann:word);
begin
  with mausreg do begin
    ax:=19;
    dx:=abwann;
    intr($33,mausreg);
  end;
end;

procedure mausinterruptaustausch(var maske:word;neu:pointer);
begin
  with mausreg do begin
    ax:=20;
    cx:=maske;
    bx:=seg(neu^);
    dx:=ofs(neu^);
    intr($33,mausreg);
    maske:=cx;
    neu:=ptr(bx,dx);
  end;
end;

procedure maussetseite(seite:word);
begin
  with mausreg do begin
    ax:=29;
    bx:=seite;
    intr($33,mausreg);
  end;
end;

function mausgetseite:word;
begin
  with mausreg do begin
    ax:=30;
    intr($33,mausreg);
    mausgetseite:=bx;
  end;
end;

function mauskeypressed:boolean;
var tasten,xpos,ypos:word;
begin
  maustastpos(tasten,xpos,ypos);
  if tasten>0 then mauskeypressed:=true
  else mauskeypressed:=false;
end;

function mausversion:word;
type wordzeiger = ^word;
begin
  with mausreg do begin
    ah:=$fa;
    bx:=0;
    intr($10,mausreg);
    if (bx=0) then mausversion:=0 else
    mausversion:=wordzeiger(ptr(es,bx))^;
  end;
end;

function mausinxy(x1,y1,x2,y2:integer):boolean;
var x,y,tasten:word;
begin
  maustastpos(tasten,x,y);
  mausinxy:=(x>=x1) and (x<=x2) and (y>=y1) and (y<=y2);
end;

end.